@@ -64,7 +64,10 @@ void usbi_free_bus(struct usbi_bus *ibus)
 
 void usbi_remove_bus(struct usbi_bus *ibus)
 {
+	pthread_mutex_lock(&usbi_buses.lock);
 	list_del(&ibus->list);
+	pthread_mutex_unlock(&usbi_buses.lock);
+	
 	usbi_free_bus(ibus);
 }
 
@@ -91,6 +94,31 @@ struct usbi_bus *usbi_find_bus_by_id(openusb_busid_t busid)
 	return NULL;
 }
 
+
+struct usbi_bus *usbi_find_bus_by_num(unsigned int busnum)
+{
+	struct usbi_bus *ibus;
+
+	/* FIXME: We should probably index the device id in
+	 * a rbtree or something
+	 */
+	pthread_mutex_lock(&usbi_buses.lock);
+	list_for_each_entry(ibus, &usbi_buses.head, list) {
+	/* safe */
+		pthread_mutex_lock(&ibus->lock);
+		if (ibus->busnum == busnum) {
+			pthread_mutex_unlock(&ibus->lock);
+			pthread_mutex_unlock(&usbi_buses.lock);
+			return ibus;
+		}
+		pthread_mutex_unlock(&ibus->lock);
+	}
+	pthread_mutex_unlock(&usbi_buses.lock);
+
+	return NULL;
+}
+
+
 static void refresh_bus(struct usbi_backend *backend)
 {
 	struct list_head busses;
@@ -172,7 +200,6 @@ void usbi_add_device(struct usbi_bus *ibus, struct usbi_device *idev)
 {
 	struct usbi_handle *handle, *thdl;
 
-	usbi_debug(NULL, 4, "add a new device");
 	/* FIXME: Handle devid rollover gracefully? */
 	idev->devid = cur_device_id++;
 
@@ -188,9 +215,7 @@ void usbi_add_device(struct usbi_bus *ibus, struct usbi_device *idev)
 
 	pthread_mutex_lock(&usbi_handles.lock);
 	list_for_each_entry_safe(handle, thdl, &usbi_handles.head, list){
-		/* every openusb instance should get notification
-		 * of this event
-		 */
+		/* every openusb instance should get notification of this event */
 		usbi_add_event_callback(handle, idev->devid, USB_ATTACH);
 	}
 	pthread_mutex_unlock(&usbi_handles.lock);
@@ -198,9 +223,13 @@ void usbi_add_device(struct usbi_bus *ibus, struct usbi_device *idev)
 
 void usbi_free_device(struct usbi_device *idev)
 {
-	if (idev->children)
+	if (idev->children) {
 		free(idev->children);
+		idev->children = NULL;
+	}
+
 	usbi_destroy_configuration(idev);
+
 	if (idev->bus->ops->free_device)
 		idev->bus->ops->free_device(idev);
 
@@ -215,9 +244,13 @@ void usbi_remove_device(struct usbi_device *idev)
 
 	openusb_devid_t devid = idev->devid;
 
+	pthread_mutex_lock(&usbi_buses.lock);
+	pthread_mutex_lock(&usbi_devices.lock);
 	list_del(&idev->bus_list);
 	list_del(&idev->dev_list);
-
+	pthread_mutex_unlock(&usbi_buses.lock);
+	pthread_mutex_unlock(&usbi_devices.lock);
+	
 	usbi_free_device(idev);
 
 	pthread_mutex_lock(&usbi_handles.lock);
@@ -295,7 +328,7 @@ int32_t openusb_get_busid_list(openusb_handle_t handle, openusb_busid_t **busids
 		return OPENUSB_NULL_LIST;
 	}
 
-	*busids = malloc((*num_busids) * sizeof (openusb_busid_t));
+	*busids = calloc((*num_busids) * sizeof (openusb_busid_t), 1);
 	if (*busids == NULL) {
 		pthread_mutex_unlock(&usbi_buses.lock);
 		usbi_debug(hdl, 2, "No resource");
@@ -512,7 +545,7 @@ int32_t openusb_get_devids_by_bus(openusb_handle_t handle, openusb_busid_t busid
 			return OPENUSB_NULL_LIST;
 		}
 
-		*devids = malloc((devcnts) * sizeof (openusb_devid_t));
+		*devids = calloc((devcnts) * sizeof (openusb_devid_t), 1);
 		if (*devids == NULL) {
 			pthread_mutex_unlock(&usbi_devices.lock);
 			return OPENUSB_NO_RESOURCES;
@@ -553,7 +586,7 @@ int32_t openusb_get_devids_by_bus(openusb_handle_t handle, openusb_busid_t busid
 		return OPENUSB_NULL_LIST;
 	}
 
-	*devids = malloc(devcnts * sizeof (openusb_devid_t));
+	*devids = calloc(devcnts * sizeof (openusb_devid_t), 1);
 	if (*devids == NULL) {
 		pthread_mutex_unlock(&ibus->devices.lock);
 		return OPENUSB_NO_RESOURCES;
@@ -640,7 +673,7 @@ int32_t openusb_get_devids_by_vendor(openusb_handle_t handle, int32_t vendor,
 		return OPENUSB_NULL_LIST;
 	}
 
-	*devids = malloc((*num_devids) * sizeof (openusb_devid_t));
+	*devids = calloc((*num_devids) * sizeof (openusb_devid_t), 1);
 	if (*devids == NULL) {
 		pthread_mutex_unlock(&usbi_devices.lock);
 		return OPENUSB_NO_RESOURCES;
@@ -717,7 +750,7 @@ int32_t openusb_get_devids_by_class(openusb_handle_t handle, int16_t devclass,
 		return OPENUSB_NULL_LIST;
 	}
 
-	*devids = malloc((*num_devids) * sizeof (openusb_devid_t));
+	*devids = calloc((*num_devids) * sizeof (openusb_devid_t), 1);
 	if (*devids == NULL) {
 		pthread_mutex_unlock(&usbi_devices.lock);
 		return OPENUSB_NO_RESOURCES;
@@ -822,10 +855,8 @@ int32_t openusb_get_raw_desc(openusb_handle_t handle,
 
 void openusb_free_raw_desc(uint8_t *buffer)
 {
-	if (buffer == NULL)
-		return;
-
 	free(buffer);
+	return;
 }
 
 int32_t openusb_parse_device_desc(openusb_handle_t handle,
@@ -833,7 +864,7 @@ int32_t openusb_parse_device_desc(openusb_handle_t handle,
 	usb_device_desc_t *devdesc)
 {
 	struct usbi_handle *hdl;
-	uint8_t *tmpbuf;
+	uint8_t *tmpbuf = NULL;
 	uint16_t tmplen;
 	int ret = OPENUSB_SUCCESS;
 	uint32_t count;
@@ -877,7 +908,7 @@ int32_t openusb_parse_config_desc(openusb_handle_t handle,
 	uint8_t cfgidx, usb_config_desc_t *cfgdesc)
 {
 	struct usbi_handle *hdl;
-	uint8_t *tmpbuf;
+	uint8_t *tmpbuf = NULL;
 	uint16_t tmplen;
 	int ret = OPENUSB_SUCCESS;
 	uint32_t count;
@@ -1195,15 +1226,15 @@ int usbi_get_string_simple(openusb_dev_handle_t dev, int index, char *buf,
 int32_t openusb_get_device_data(openusb_handle_t handle, openusb_devid_t devid,
 	uint32_t flags, openusb_dev_data_t **data)
 {
-	openusb_dev_data_t *pdata;
-	struct usbi_handle *plib;
-	struct usbi_device *pdev;
+	openusb_dev_data_t *pdata = NULL;
+	struct usbi_handle *plib = NULL;
+	struct usbi_device *pdev = NULL;
 	uint16_t datalen;
-	uint8_t *descdata;
+	uint8_t *descdata = NULL;
 	int ret;
 	char strings[256];
 	openusb_dev_handle_t hdev;
-	struct usbi_dev_handle *devh, *dev_found=NULL;
+	struct usbi_dev_handle *devh = NULL, *dev_found=NULL;
 
 	usbi_debug(NULL, 4, "devid=%d, flags=%d",(int)devid, flags);
 
@@ -1310,7 +1341,7 @@ int32_t openusb_get_device_data(openusb_handle_t handle, openusb_devid_t devid,
 			/* this should not be an error, perhaps we just don't have permission */
 			pdata->manufacturer = NULL;
 		} else {
-			if ((pdata->manufacturer = malloc(strings[0])) == NULL) {
+			if ((pdata->manufacturer = calloc(strings[0], 1)) == NULL) {
 				free(pdata);
 				if (!dev_found) { openusb_close_device(hdev); }
 				return OPENUSB_NO_RESOURCES;
@@ -1327,7 +1358,7 @@ int32_t openusb_get_device_data(openusb_handle_t handle, openusb_devid_t devid,
 			/* this should not be an error, perhaps we just don't have permission */
 			pdata->product = NULL;
 		} else {
-			if ((pdata->product= malloc(strings[0])) == NULL) {
+			if ((pdata->product= calloc(strings[0], 1)) == NULL) {
 				free(pdata->manufacturer);
 				free(pdata);
 				if (!dev_found) { openusb_close_device(hdev); }
@@ -1344,7 +1375,7 @@ int32_t openusb_get_device_data(openusb_handle_t handle, openusb_devid_t devid,
 			/* this should not be an error, perhaps we just don't have permission */
 			pdata->serialnumber = NULL;
 		} else {
-			if ((pdata->serialnumber = malloc(strings[0])) == NULL) {
+			if ((pdata->serialnumber = calloc(strings[0], 1)) == NULL) {
 				free(pdata->product);
 				free(pdata->manufacturer);
 				free(pdata);
@@ -1362,22 +1393,22 @@ int32_t openusb_get_device_data(openusb_handle_t handle, openusb_devid_t devid,
 
 get_raw:
 	ret = openusb_get_raw_desc(handle, devid, USB_DESC_TYPE_CONFIG,
-			pdev->cur_config, 0, &descdata, &datalen);
+			pdev->cur_config_index, 0, &descdata, &datalen);
 	if (ret != 0) {
 		usbi_debug(NULL, 1, "Get raw config(%d) desc fail",
-			pdev->cur_config);
+			pdev->cur_config_value);
 		goto fail;
 	}
 
 	ret = openusb_parse_config_desc(handle, devid, descdata, datalen,
-			pdev->cur_config, &pdata->cfg_desc);
+			pdev->cur_config_value, &pdata->cfg_desc);
 	if (ret != 0) {
 		usbi_debug(NULL, 1, "Parse config fail");
 		goto fail;
 	}
 	usbi_debug(NULL, 4, "data len = %d",datalen);
 
-	pdata->raw_cfg_desc = malloc(datalen);
+	pdata->raw_cfg_desc = calloc(datalen, 1);
 	if (!pdata->raw_cfg_desc) {
 		openusb_free_raw_desc(descdata);
 		ret = OPENUSB_NO_RESOURCES;
@@ -1402,6 +1433,7 @@ get_raw:
 	return 0;
 
 fail:
+	if (!dev_found) { openusb_close_device(hdev); }
 	free(pdata->product);
 	free(pdata->manufacturer);
 	free(pdata->serialnumber);
@@ -1472,3 +1504,38 @@ struct usbi_list *usbi_get_devices_list(void)
 {
 	return (&usbi_devices);
 }
+
+/* search for the configuration value of a specified configuration index */
+uint8_t usbi_get_cfg_value_by_index(struct usbi_dev_handle *hdev, int cfgndx)
+{
+	uint8_t cfgval = 0;
+
+	if (hdev != NULL && cfgndx >= 0
+		&& usbi_fetch_and_parse_descriptors(hdev) == 0
+		&& hdev->idev->desc.num_configs > cfgndx) {
+
+		cfgval = hdev->idev->desc.configs[cfgndx].desc.bConfigurationValue;
+	}
+
+	return cfgval;
+}
+
+/* search for the descriptor index of a specified configuration value */
+int usbi_get_cfg_index_by_value(struct usbi_dev_handle *hdev, uint8_t cfgval)
+{
+	int cfgndx = -1;
+	int i;
+
+	if (hdev != NULL && usbi_fetch_and_parse_descriptors(hdev) == 0) {
+		for (i = 0; i < hdev->idev->desc.num_configs; i++) {
+			if (hdev->idev->desc.configs[i].desc.bConfigurationValue
+				== cfgval) {
+
+				cfgndx = i;
+				break;
+			}
+		}
+	}
+
+	return cfgndx;
+}
