@@ -23,6 +23,12 @@
 
 #define	USB_MAX_DEBUG_LEVEL	5
 
+struct usbi_list usbi_handles; /* protected by usbi_handles.lock */
+struct usbi_list usbi_dev_handles; /* protected by usbi_dev_handles.lock*/
+
+struct usbi_list usbi_buses; /* protected by usbi_buses.lock */
+struct usbi_list usbi_devices; /* protected by usbi_device.lock */
+
 /*
  * env variables:
  *	OPENUSB_BACKEND_PATH - file path of backends
@@ -30,7 +36,6 @@
  */
 
 static int32_t openusb_global_debug_level = 0;
-
 static openusb_handle_t cur_handle = 1; /* protected by usbi_lock */
 static openusb_dev_handle_t cur_dev_handle = 1; /* protected by usbi_lock */
 
@@ -48,6 +53,9 @@ static struct usbi_list event_callbacks;
 /* the background thread processing events */
 static pthread_t event_callback_thread;
 static pthread_cond_t event_callback_cond;
+static volatile int32_t event_callback_exit = 0;
+
+
 
 void _usbi_debug(struct usbi_handle *hdl, uint32_t level, const char *func,
 	uint32_t line, char *fmt, ...)
@@ -118,13 +126,12 @@ void usbi_add_event_callback(struct usbi_handle *hdl, openusb_devid_t devid,
 	list_init(&cb->list);
 
 	pthread_mutex_lock(&event_callbacks.lock);
-	
+
 	list_add(&cb->list, &event_callbacks.head);
 
 	pthread_cond_signal(&event_callback_cond);
-	
-	if (callback_queue_full == 0)
-		callback_queue_full = 1;
+
+	callback_queue_full++;
 
 	pthread_mutex_unlock(&event_callbacks.lock);
 }
@@ -138,8 +145,13 @@ static void *process_event_callbacks(void *unused)
 		pthread_mutex_lock(&event_callbacks.lock);
 
 		while(callback_queue_full == 0) {
-			pthread_cond_wait(&event_callback_cond, 
-					&event_callbacks.lock);
+			pthread_cond_wait(&event_callback_cond, &event_callbacks.lock);
+			if (event_callback_exit) {
+				/* we're being told we need to shutdown, reset our flag and exit */
+				event_callback_exit = 0;
+				pthread_mutex_unlock(&event_callbacks.lock);
+				return (NULL);
+			}
 		}
 
 		/*
@@ -156,7 +168,10 @@ static void *process_event_callbacks(void *unused)
 
 			cb = list_entry(&(listh->prev), struct eventcallback,
 				list);
-
+			
+			/*Get the pointer to the next list element, then we can delete this one*/
+			listh = listh->next;
+			
 			list_del(&cb->list);
 
 			devid = cb->devid;
@@ -180,14 +195,12 @@ static void *process_event_callbacks(void *unused)
 
 			pthread_mutex_lock(&event_callbacks.lock);
 
-			listh = listh->next;
-
 			/* don't reference any element of cb after this */
 			free(cb);
-		}
 
-		/* the list should be empty here */
-		callback_queue_full = 0;
+			/* the list should be empty here */
+			callback_queue_full--;
+		}
 
 		pthread_mutex_unlock(&event_callbacks.lock);
 	}
@@ -227,7 +240,7 @@ static int load_backend(const char *filepath)
 		goto err;
 	}
 
-	backend = malloc(sizeof(*backend));
+	backend = calloc(sizeof(*backend), 1);
 	if (!backend) {
 		fprintf(stderr, "couldn't allocate memory for backend\n");
 		goto err;
@@ -405,7 +418,9 @@ static int usbi_init_common(void)
 
 	pthread_cond_init(&event_callback_cond, NULL);
 
-	/* Start up thread for callbacks */
+	/* Start up thread for callbacks, make sure our exit flag is 0,
+	 * if we're creating the thread we definitely don't want it to exit */
+	event_callback_exit = 0;
 	ret = pthread_create(&event_callback_thread, NULL,
 		process_event_callbacks, NULL);
 	if (ret < 0) {
@@ -451,8 +466,11 @@ static void usbi_fini_common()
 {
 	/* XXX need to free device, bus and backend list */
 
-	/* shared thread and cond should be destroyed ? */
-
+	/* first we need to make sure that the event callback thread is shutdown */
+	event_callback_exit = 1;
+	pthread_cond_signal(&event_callback_cond);
+	pthread_join(event_callback_thread, NULL);
+	
 	pthread_cond_destroy(&event_callback_cond);
 	usbi_list_fini(&event_callbacks);
 	usbi_list_fini(&usbi_dev_handles);
@@ -651,9 +669,16 @@ void openusb_fini(openusb_handle_t handle)
 		usbi_debug(NULL, 4, "Last lib handle");
 		usbi_fini_common();
 
-		/* last openusb instance, destroy lock */
+		/* last openusb instance, DON'T destroy it */
 		pthread_mutex_unlock(&usbi_lock);
-		pthread_mutex_destroy(&usbi_lock);
+
+		/* last openusb instance, unload the backends */
+		list_for_each_entry_safe(backend, tbackend, &backends, list) {
+		/* safe */
+			dlclose(backend->handle);	/* shutdown the backend */
+			list_del(&backend->list);	/* remove the backend from the list */
+		}
+
 		return;
 	}
 	pthread_mutex_unlock(&usbi_lock);
@@ -762,7 +787,7 @@ int32_t openusb_set_default_timeout(openusb_handle_t handle,
 
 /* find an opened device's usbi_dev_handle by its openusb_dev_handle */
 struct usbi_dev_handle *usbi_find_dev_handle(openusb_dev_handle_t dev)
-{ 
+{
 	struct usbi_dev_handle *hdev;
 
 	/* fail if openusb is not inited */
@@ -774,7 +799,7 @@ struct usbi_dev_handle *usbi_find_dev_handle(openusb_dev_handle_t dev)
 	pthread_mutex_unlock(&usbi_lock);
 
 	/* FIXME: We should probably index the device id in a rbtree or
-	 * something 
+	 * something
 	 */
 	pthread_mutex_lock(&usbi_dev_handles.lock);
 	list_for_each_entry(hdev, &usbi_dev_handles.head, list) {
@@ -789,9 +814,9 @@ struct usbi_dev_handle *usbi_find_dev_handle(openusb_dev_handle_t dev)
 		pthread_mutex_unlock(&hdev->lock);
 	}
 	pthread_mutex_unlock(&usbi_dev_handles.lock);
-	
+
 	return NULL;
-}   
+}
 
 /* find a device's usbi_device struct by its devid */
 struct usbi_device *usbi_find_device_by_id(openusb_devid_t devid)
@@ -832,6 +857,7 @@ int32_t openusb_open_device(openusb_handle_t handle, openusb_devid_t devid,
 	struct usbi_handle *hdl; /* lib internal handle */
 	struct usbi_device *idev;
 	struct usbi_dev_handle *hdev;
+	uint8_t cfg;
 	int ret;
 	int i;
 	
@@ -849,7 +875,7 @@ int32_t openusb_open_device(openusb_handle_t handle, openusb_devid_t devid,
 	if (!idev)
 		return OPENUSB_UNKNOWN_DEVICE;
 
-	hdev = malloc(sizeof (*hdev));
+	hdev = calloc(sizeof (*hdev), 1);
 	if (!hdev)
 		return OPENUSB_NO_RESOURCES;
 
@@ -903,6 +929,12 @@ int32_t openusb_open_device(openusb_handle_t handle, openusb_devid_t devid,
 
 	pthread_mutex_unlock(&usbi_dev_handles.lock);
 
+	/* get current device configuration value */
+	ret = openusb_get_configuration(*dev, &cfg);
+	if (ret != OPENUSB_SUCCESS) {
+		openusb_close_device(*dev);
+		return ret;
+	}
 
 	return OPENUSB_SUCCESS;
 }
